==> touhoulittlemaid-fabric-1.21.1-0.0.2-neo1.2.2-sources.jar: com/github/tartaricacid/touhoulittlemaid/entity/item/EntityBox.java <==
package com.github.tartaricacid.touhoulittlemaid.entity.item;

import com.github.tartaricacid.touhoulittlemaid.init.InitSounds;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1293;
import net.minecraft.class_1294;
import net.minecraft.class_1297;
import net.minecraft.class_1299;
import net.minecraft.class_1311;
import net.minecraft.class_1313;
import net.minecraft.class_1321;
import net.minecraft.class_1657;
import net.minecraft.class_1802;
import net.minecraft.class_1937;
import net.minecraft.class_243;
import net.minecraft.class_2487;
import net.minecraft.class_2940;
import net.minecraft.class_2943;
import net.minecraft.class_2945;
import net.minecraft.class_3532;
import net.minecraft.class_4050;
import net.minecraft.world.entity.*;

public class EntityBox extends class_1297 {
    public static final int FIRST_STAGE = 64;
    public static final int SECOND_STAGE = 60;
    public static final int THIRD_STAGE = 0;
    public static final int MAX_TEXTURE_SIZE = 8;
    private static final class_2940<Integer> OPEN_STAGE = class_2945.method_12791(EntityBox.class, class_2943.field_13327);
    private static final class_2940<Integer> TEXTURE_INDEX = class_2945.method_12791(EntityBox.class, class_2943.field_13327);
    public static final class_1299<EntityBox> TYPE = class_1299.class_1300.<EntityBox>method_5903(EntityBox::new, class_1311.field_17715)
            .method_17687(2.0f, 2.0f).method_27299(10).method_5905("box");
    private static final String STAGE_TAG = "OpenStage";
    private static final String TEXTURE_TAG = "TextureIndex";

    public EntityBox(class_1299<?> entityTypeIn, class_1937 worldIn) {
        super(entityTypeIn, worldIn);
        this.setRandomTexture();
    }

    public EntityBox(class_1937 worldIn) {
        this(TYPE, worldIn);
    }

    @Override
    public void method_5670() {
        if (!this.method_5740() && !this.method_24828()) {
            this.method_5784(class_1313.field_6308, this.method_18798().method_1031(0, -0.1, 0));
        }
        super.method_5670();
        int stage = getOpenStage();
        this.stageChange(stage);
        this.method_5685().stream().filter(e -> e instanceof class_1321)
                .forEach(e -> applyInvisibilityEffect((class_1321) e, stage));
    }

    @Override
    public class_1269 method_5688(class_1657 player, class_1268 hand) {
        if (this.getOpenStage() == FIRST_STAGE) {
            setOpenStage(FIRST_STAGE - 1);
            this.method_5783(InitSounds.BOX_OPEN, 3.0f, 1.0f);
            return class_1269.method_29236(method_37908().field_9236);
        }
        if (this.getOpenStage() == SECOND_STAGE) {
            setOpenStage(SECOND_STAGE - 1);
            this.method_5783(InitSounds.BOX_OPEN, 3.0f, 1.0f);
            return class_1269.method_29236(method_37908().field_9236);
        }
        if (this.getOpenStage() == THIRD_STAGE) {
            setOpenStage(THIRD_STAGE - 1);
            this.method_5783(InitSounds.BOX_OPEN, 3.0f, 2.0f);
            return class_1269.method_29236(method_37908().field_9236);
        }
        return super.method_5688(player, hand);
    }

    @Override
    protected void method_5693(class_2945.class_9222 builder) {
        builder.method_56912(OPEN_STAGE, FIRST_STAGE);
        builder.method_56912(TEXTURE_INDEX, 0);
    }

    @Override
    protected void method_5749(class_2487 compound) {
        if (compound.method_10545(STAGE_TAG)) {
            setOpenStage(compound.method_10550(STAGE_TAG));
        }
        if (compound.method_10545(TEXTURE_TAG)) {
            setTextureIndex(compound.method_10550(TEXTURE_TAG));
        }
    }

    @Override
    protected void method_5652(class_2487 compound) {
        compound.method_10569(STAGE_TAG, getOpenStage());
        compound.method_10569(TEXTURE_TAG, getTextureIndex());
    }

    @Override
    public boolean method_30948() {
        return this.method_5805();
    }

    @Override
    public boolean method_5863() {
        return this.method_5805();
    }

    public int getOpenStage() {
        return this.field_6011.method_12789(OPEN_STAGE);
    }

    public void setOpenStage(int stage) {
        this.field_6011.method_12778(OPEN_STAGE, class_3532.method_15340(stage, -1, FIRST_STAGE));
    }

    public int getTextureIndex() {
        return this.field_6011.method_12789(TEXTURE_INDEX);
    }

    public void setTextureIndex(int index) {
        this.field_6011.method_12778(TEXTURE_INDEX, class_3532.method_15340(index, 1, MAX_TEXTURE_SIZE - 1));
    }

    public void setRandomTexture() {
        setTextureIndex(field_5974.method_43048(MAX_TEXTURE_SIZE));
    }

    private void stageChange(int stage) {
        if (stage != FIRST_STAGE && stage != SECOND_STAGE && stage != THIRD_STAGE) {
            this.setOpenStage(stage - 1);
        }
        if (stage < THIRD_STAGE) {
            this.method_31472();
            if (!this.field_6002.field_9236) {
                this.method_5870(class_1802.field_8407, 2 + field_5974.method_43048(3));
            }
        }
    }

    private void applyInvisibilityEffect(class_1321 tameable, int stage) {
        if (stage >= FIRST_STAGE) {
            tameable.method_6092(new class_1293(class_1294.field_5905, 2, 1, false, false));
        }
    }

    @Override
    public class_243 method_52538(class_1297 pEntity) {
        return this.method_19538().method_1019(this.method_52533(pEntity, this.method_18377(class_4050.field_18076), 1.0F)).method_1023(0, 2, 0);
    }
}

==> touhoulittlemaid-fabric-1.21.1-0.1.7.2-neo1.3.8-sources.jar: com/github/tartaricacid/touhoulittlemaid/entity/item/EntityBox.java <==
package com.github.tartaricacid.touhoulittlemaid.entity.item;

import com.github.tartaricacid.touhoulittlemaid.init.InitSounds;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1293;
import net.minecraft.class_1294;
import net.minecraft.class_1297;
import net.minecraft.class_1299;
import net.minecraft.class_1311;
import net.minecraft.class_1313;
import net.minecraft.class_1321;
import net.minecraft.class_1657;
import net.minecraft.class_173;
import net.minecraft.class_181;
import net.minecraft.class_1937;
import net.minecraft.class_2398;
import net.minecraft.class_243;
import net.minecraft.class_2487;
import net.minecraft.class_2940;
import net.minecraft.class_2943;
import net.minecraft.class_2945;
import net.minecraft.class_3218;
import net.minecraft.class_3532;
import net.minecraft.class_4050;
import net.minecraft.class_52;
import net.minecraft.class_5321;
import net.minecraft.class_8567;
import net.minecraft.world.entity.*;

public class EntityBox extends class_1297 {
    public static final int FIRST_STAGE = 0;
    public static final int SECOND_STAGE = 1;
    public static final int THIRD_STAGE = 2;
    public static final int MAX_TEXTURE_SIZE = 8;
    private static final class_2940<Integer> OPEN_STAGE = class_2945.method_12791(EntityBox.class, class_2943.field_13327);
    private static final class_2940<Integer> TEXTURE_INDEX = class_2945.method_12791(EntityBox.class, class_2943.field_13327);
    public static final class_1299<EntityBox> TYPE = class_1299.class_1300.<EntityBox>method_5903(EntityBox::new, class_1311.field_17715)
            .method_17687(2.0f, 2.0f).method_27299(10).method_5905("box");
    private static final String STAGE_TAG = "OpenStage";
    private static final String TEXTURE_TAG = "TextureIndex";

    // 用于第二阶段蛋糕盒动画计算的变量
    public long thirdStageTimeStamp = 0L;
    private int thirdStageTicks = 0;

    public EntityBox(class_1299<?> entityTypeIn, class_1937 worldIn) {
        super(entityTypeIn, worldIn);
        this.setRandomTexture();
    }

    public EntityBox(class_1937 worldIn) {
        this(TYPE, worldIn);
    }

    @Override
    public void method_5670() {
        if (!this.method_5740() && !this.method_24828()) {
            this.method_5784(class_1313.field_6308, this.method_18798().method_1031(0, -0.1, 0));
        }
        super.method_5670();

        int stage = this.getOpenStage();
        if (stage == FIRST_STAGE) {
            this.method_5685().stream().filter(e -> e instanceof class_1321)
                    .forEach(e -> applyInvisibilityEffect((class_1321) e));
        } else if (stage == THIRD_STAGE) {
            thirdStageTicks++;
            if (thirdStageTicks > 100) {
                this.addStageChange();
            }
        }
    }

    @Override
    public class_1269 method_5688(class_1657 player, class_1268 hand) {
        this.addStageChange();
        if (this.getOpenStage() == SECOND_STAGE) {
            this.method_5783(InitSounds.BOX_OPEN, 3.0f, 1.0f);
            return class_1269.method_29236(method_37908().field_9236);
        }
        if (this.getOpenStage() == THIRD_STAGE) {
            this.thirdStageTimeStamp = System.currentTimeMillis();
            this.method_5783(InitSounds.BOX_OPEN, 3.0f, 1.0f);
            return class_1269.method_29236(method_37908().field_9236);
        }
        this.method_5783(InitSounds.BOX_OPEN, 3.0f, 2.0f);
        return class_1269.method_29236(method_37908().field_9236);
    }

    @Override
    protected void method_5693(class_2945.class_9222 builder) {
        builder.method_56912(OPEN_STAGE, FIRST_STAGE);
        builder.method_56912(TEXTURE_INDEX, 0);
    }

    @Override
    protected void method_5749(class_2487 compound) {
        if (compound.method_10545(STAGE_TAG)) {
            setOpenStage(compound.method_10550(STAGE_TAG));
        }
        if (compound.method_10545(TEXTURE_TAG)) {
            setTextureIndex(compound.method_10550(TEXTURE_TAG));
        }
    }

    @Override
    protected void method_5652(class_2487 compound) {
        compound.method_10569(STAGE_TAG, getOpenStage());
        compound.method_10569(TEXTURE_TAG, getTextureIndex());
    }

    @Override
    public boolean method_30948() {
        return this.method_5805();
    }

    @Override
    public boolean method_5863() {
        return this.method_5805();
    }

    public int getOpenStage() {
        return this.field_6011.method_12789(OPEN_STAGE);
    }

    private void setOpenStage(int stage) {
        this.field_6011.method_12778(OPEN_STAGE, stage);
    }

    public int getTextureIndex() {
        return this.field_6011.method_12789(TEXTURE_INDEX);
    }

    private void setTextureIndex(int index) {
        this.field_6011.method_12778(TEXTURE_INDEX, class_3532.method_15340(index, 1, MAX_TEXTURE_SIZE - 1));
    }

    public void setRandomTexture() {
        setTextureIndex(field_5974.method_43048(MAX_TEXTURE_SIZE));
    }

    private void addStageChange() {
        this.setOpenStage(this.getOpenStage() + 1);
        if (this.getOpenStage() > THIRD_STAGE) {
            this.method_5768();
        }
    }

    @Override
    public void method_5768() {
        if (this.field_6002 instanceof class_3218 serverLevel) {
            serverLevel.method_14199(class_2398.field_11236, this.method_23317(), this.method_23318() + 0.25, this.method_23321(),
                    20, 1, 1, 1, 0.2);
            class_5321<class_52> table = this.method_5864().method_16351();
            class_52 lootTable = serverLevel.method_8503().method_58576().method_58295(table);
            class_8567.class_8568 builder = new class_8567.class_8568(serverLevel)
                    .method_51874(class_181.field_1226, this)
                    .method_51874(class_181.field_24424, this.method_19538())
                    .method_51874(class_181.field_1231, method_48923().method_51847());
            class_8567 params = builder.method_51875(class_173.field_1173);
            lootTable.method_51880(params, 0, this::method_5775);
        }
        super.method_5768();
    }

    private void applyInvisibilityEffect(class_1321 tameable) {
        tameable.method_6092(new class_1293(class_1294.field_5905, 2, 1, false, false));
    }

    @Override
    public class_243 method_52538(class_1297 pEntity) {
        return this.method_19538().method_1019(this.method_52533(pEntity, this.method_18377(class_4050.field_18076), 1.0F)).method_1023(0, 2, 0);
    }
}
